# -*- coding: utf-8 -*-
import six


class BaseError(Exception):
    def __init__(self, message=None, codigo=None, message_values=()):
        self.message_values = message_values
        self.codigo = codigo
        if six.PY2:
            super(Exception, self).__init__(message)
        else:
            super().__init__(message)


class InvalidParam(BaseError):
    def __init__(self, message="MitteProError - Parâmetro {0} é inválido. Razão: {1}", codigo=None, message_values=()):
        self.message_values = message_values
        self.codigo = codigo
        if message_values:
            message = message.format(*message_values)
        if six.PY2:
            super(InvalidParam, self).__init__(message)
        else:
            super().__init__(message)


class APIError(BaseError):
    def __init__(self, message="MitteProError. Razão: {0}", codigo=None, message_values=()):
        self.message_values = message_values
        self.codigo = codigo
        if message_values:
            message = message.format(*message_values)
        if six.PY2:
            super(APIError, self).__init__(message)
        else:
            super().__init__(message)


class TimeoutError(BaseError):
    def __init__(self, message="MitteProError. Razão: O servidor não respondeu dentro do tempo que você estipulou. "
                               "O tempo foi de {0} segundo(s)", codigo=None, message_values=()):
        self.message_values = message_values
        self.codigo = codigo
        if message_values:
            message = message.format(*message_values)
        if six.PY2:
            super(TimeoutError, self).__init__(message)
        else:
            super().__init__(message)


class ImproperlyConfigured(BaseError):
    def __init__(self, message="MitteProError. Configuração inapropriada. Razão: {0}", codigo=None, message_values=()):
        self.message_values = message_values
        self.codigo = codigo
        if message_values:
            message = message.format(*message_values)
        if six.PY2:
            super(ImproperlyConfigured, self).__init__(message)
        else:
            super().__init__(message)



# coding=utf-8
try:
    from test_variables import server_uri_test
except ImportError:
    server_uri_test = False
try:
    from test_variables import search_variables
except ImportError:
    search_variables = False
try:
    from test_variables import variables
except ImportError:
    variables = False
import os
import base64
import unittest
from client import MittePro
from models import Mail, SearchMailArgs


class TestMittePro(unittest.TestCase):
    def setUp(self):
        self.server_uri_test = None
        self.variables = {
            "recipients": [
                "Foo Bar <foo.bar@gmail.com>",
                "Fulano <fulano@gmail.com>",
                "<ciclano@gmail.com>"
            ],
            "context_per_recipient": {
                "foo.bar@gmail.com": {"foo": True},
                "fulano@gmail.com.br": {"bar": True}
            },
            "from_": 'Beutrano <beutrano@mail.com>',
            "from_2": '<beutrano@mail.com>',
            "template_slug": 'test-101',
            "message_text": "Using this message instead.",
            "message_html": "<em>Using this message <strong>instead</strong>.</em>",
            "key": '2e7be7ced03535958e35',
            "secret": 'ca3cdba202104fd88d01',
            "files_names": [
                # 'foo.pdf',
                # 'bar.jpg',
                # 'foo_bar.txt',
            ]
        }
        self.search_variables = {
            'app_ids': '1001',
            'start': '2017-10-26',
            'end': '2017-10-27',
            'uuids': [
                '21da05e09a214bf',
                '7b9332128a3f461',
                '09f7ceac90fe4b3',
                '0f39a611031c4ff',
                'f2412b7062814de'
            ]
        }

        if variables:
            self.variables = variables
        if server_uri_test:
            self.server_uri_test = server_uri_test
        if search_variables:
            self.search_variables = search_variables

        self.mittepro = MittePro(key=self.variables['key'], secret=self.variables['secret'], fail_silently=False,
                                 server_uri=self.server_uri_test, timeout_read=20)

    def get_attachments(self):
        attachments = []
        files = self.variables['files_names']
        for dfile in files:
            content = base64.encodebytes(open(
                os.path.join(os.path.expanduser('~'), 'test_files', dfile), 'rb'
            ).read())
            attachments.append({'file': content, 'name': dfile})
        return attachments

    def test_method_post_text(self):
        # attachments = []
        # attachments = self.get_attachments()
        mail = Mail(
            track_open=False,
            track_html_link=False,
            track_text_link=True,
            activate_tracking=False,
            subject='Mittepro-py client test',
            # send_at='2019-05-04 08:00:00',
            message_text=self.variables['message_text'],
            message_html=self.variables['message_html'],
            from_=self.variables['from_'],
            recipient_list=self.variables['recipients'],
            # context={'GMERGE': 'Mah oia Soh'},
            # context_per_recipient=self.variables['']
        )
        try:
            response = self.mittepro.send(mail)
            if response and 'emails_enviados' in response:
                self.assertGreater(len(response['emails_enviados']), 0)
            else:
                self.assertIsNotNone(response)
        except Exception as e:
            print("Exception", e)

    def t2est_method_post_template(self):
        # attachments = []
        # attachments = self.get_attachments()
        mail = Mail(
            track_open=True,
            track_html_link=False,
            track_text_link=False,
            activate_tracking=True,
            subject='Mittepro-py client test',
            send_at='2019-05-04 08:00:00',
            from_=self.variables['from_'],
            recipient_list=self.variables['recipients'],
            template_slug='tpl-teste',
            use_tpl_default_name=True,
            use_tpl_default_email=True,
            use_tpl_default_subject=True,
            context={'GMERGE': 'Mah oia Soh'},
            context_per_recipient=self.variables['context_per_recipient'],
            # attachments=attachments
        )
        # print mail.get_payload()
        response = self.mittepro.send_template(mail)
        print("response", response)
        if response and 'emails_enviados' in response:
            self.assertGreater(len(response['emails_enviados']), 0)
        else:
            self.assertIsNotNone(response)

    def t2est_method_get_mail_search(self):
        search_args = SearchMailArgs(
            end=self.search_variables['end'],
            start=self.search_variables['start'],
            app_cods=self.search_variables['app_ids'],
            # name_sender=self.search_variables['name_sender'],
            # email_sender=self.search_variables['email_sender'],
            # name_receiver=self.search_variables['name_receiver'],
            # template_slug=self.search_variables['template_slug'],
            # email_receiver=self.search_variables['email_receiver'],
        )
        response = self.mittepro.mail_search(search_args)
        if response and 'qtd_mails' in response and response['qtd_mails'] > 0:
            print("qtd_mails %s" % response['qtd_mails'])
            self.assertGreater(response['qtd_mails'], 0)
        else:
            print("Nothing found")
            self.assertIsNotNone(response)

    def t2est_method_get_mail_search_by_ids(self):
        response = self.mittepro.mail_search_by_ids(self.search_variables['uuids'])
        if response and len(response) > 0:
            print("uuids %s" % self.search_variables['uuids'])
            print("len(response) %s" % len(response))
            self.assertGreater(len(response), 0)
        else:
            print("Nothing found")
            self.assertIsNotNone(response)

if __name__ == '__main__':
    unittest.main()



# coding=utf-8

variables = {
    "recipients": [
        "Thíàgõ <thiago.decastro2@gmail.com>",
        "Läçêrdà <thiago.dsn.cir@alterdata.com.br>"
    ],
    "context_per_recipient": {
        "thiago.decastro2@gmail.com": {
            "updateprofile": "https://www.google.com/doodles/celebrating-ruth-asawa",
            "update_profile": "https://www.google.com/doodles/teachers-day-2019-paraguay",
            "sobrenome": "Tíâò",
            "lastname": "Làçẽrdä",
            "forward": "https://www.google.com/doodles/celebrating-the-new-era",
            "resub": "https://www.google.com/doodles/na-hye-soks-123rd-birthday",
            "unsub": "https://www.google.com/doodles/rosy-afsaris-73rd-birthday",
            "aniversario": "17/11",
            "birthday": "12/01",
            "e-mail": "thiago.dsn.cir@alterdata.com.br",
            "email": "alex.dsn.cir@alterdata.com.br",
            "nome": "Àléx",
            "name": "Ãlëx",
        },
        "thiago.dsn.cir@alterdata.com.br": {
            "updateprofile": "https://www.google.com/doodles/celebrating-ruth-asawa",
            "update_profile": "https://www.google.com/doodles/teachers-day-2019-paraguay",
            "sobrenome": "Tíâò",
            "lastname": "Làçẽrdä",
            "forward": "https://www.google.com/doodles/celebrating-the-new-era",
            "resub": "https://www.google.com/doodles/na-hye-soks-123rd-birthday",
            "unsub": "https://www.google.com/doodles/rosy-afsaris-73rd-birthday",
            "aniversario": "17/11",
            "birthday": "12/01",
            "e-mail": "thiago.dsn.cir@alterdata.com.br",
            "email": "alex.dsn.cir@alterdata.com.br",
            "nome": "Àléx",
            "name": "Ãlëx",
        }
    },
    "from_": 'Beutrano <beutrano@alterdata.com.br>',
    "from_2": '<beutrano@mail.com>',
    "template_slug": 'test-101',
    "message_text": "Using this message instead.",
    "message_html": "<em>Using this message <strong>instead</strong>.</em>",
    "key": '1e4be7cdd03545958e34',
    "secret": 'cf8cdba282104ed88f0a'
}
server_uri_test = 'http://172.16.72.93:8002'

search_variables = {
    'app_ids': '1001',
    'status': ['1', '2'],
    'start': '2019-05-01',
    'end': '2019-05-03',
    'name_sender': 'Beutrano',
    'email_sender': 'beutrano@alterdata.com.br',
    'name_receiver': 'Läçêrdà',
    'email_receiver': 'thiago.dsn.cir@alterdata.com.br',
    'template_slug': 'tpl-teste',
    'uuids': [
        '21da05e09a214bf',
        '7b9332128a3f461',
        '09f7ceac90fe4b3',
        '0f39a611031c4ff',
        'f2412b7062814de'
    ]
}



import six
import hmac
import arrow
import hashlib
import datetime
import collections
from ..utils import merge_two_dicts
from .query_encoder import QueryEncoder


class ArgumentError(Exception):
    pass


class Token(object):

    key = None
    secret = None

    def __init__(self, key, secret):
        self.key = key
        self.secret = secret

    def sign(self, request):
        request.sign(self)


class Request(object):

    path = None
    method = None
    signed = False
    auth_dict = None
    query_dict = None

    AUTH_VERSION = '1.0'

    ISO8601 = arrow.utcnow().format('YYYY-MM-DDTHH:mm:ssZZ')

    def __init__(self, method, path, query):
        if not isinstance(path, str):
            raise ArgumentError('Expected string')
        if not isinstance(query, dict):
            raise ArgumentError('Expected dict')

        query_dict = {}
        auth_dict = {}

        for key, value in query.items():
            key = key.lower()
            if 'auth_' in key:
                auth_dict[key] = value
            else:
                query_dict[key] = value

        self.method = method.upper()
        self.path = path
        self.query_dict = query_dict
        self.auth_dict = auth_dict
        self.signed = False

    # Sign the request with the given token, and return the computed
    # authentication parameters
    def sign(self, token, auth_timestamp=None):
        self.signed = True
        self.auth_dict = dict(
            auth_key=token.key,
            auth_version=self.AUTH_VERSION,
            auth_timestamp=auth_timestamp or arrow.utcnow().timestamp
        )
        self.auth_dict['auth_signature'] = self.signature(token)
        return self.auth_dict

    def parameter_string(self):
        param_hash = merge_two_dicts(self.query_dict, (self.auth_dict or {}))
        # Convert keys to lowercase strings
        params_dict = {}
        for key, value in param_hash.items():
            params_dict[key.lower()] = value

        # Exclude signature from signature generation!
        if 'auth_signature' in params_dict:
            del params_dict['auth_signature']

        params_dict = collections.OrderedDict(sorted(params_dict.items()))
        params_list = []
        for key, value in params_dict.items():
            params_list.append(QueryEncoder.encode_param_without_escaping(key, value))
        return '&'.join(params_list)

    def string_to_sign(self):
        return "\n".join([self.method, self.path, self.parameter_string()])

    def signature(self, token):
        msg = self.string_to_sign()
        if six.PY2: 
            return hmac.new(str(token.secret), msg, hashlib.sha256).hexdigest()
        else:
            return hmac.new(bytes(token.secret, 'utf8'), bytes(msg, 'utf8'), hashlib.sha256).hexdigest()



import six
import json
import urllib


class QueryEncoder(object):

    @staticmethod
    def encode_param(key, value):
        if isinstance(value, list):
            return '&'.join([QueryEncoder.escape(key) + '[]=' + QueryEncoder.escape(item) for item in value])
        else:
            return QueryEncoder.escape(key) + '=' + QueryEncoder.escape(value)

    @staticmethod
    def encode_param_without_escaping(key, value):
        if isinstance(value, list):
            return str('&'.join([key + '[]=' + item for item in value]))
        else:
            if isinstance(value, dict):
                value = json.dumps(value)
            elif six.PY2 and isinstance(value, unicode):
                value = str(value.encode('utf-8'))
        return "{key}={value}".format(key=key, value=value)

    @staticmethod
    def escape(s):
        if six.PY2:
            return urllib.quote(s, '')
        else:
            return urllib.parse.quote(s, '')



import json
import arrow
import logging
import requests
from utils import is_json
from mitte_exceptions import APIError, TimeoutError
from apysignature.query_encoder import QueryEncoder
from apysignature.signature import Request as Request_sig, Token
from requests import Request, Session, ReadTimeout, ConnectTimeout, HTTPError

__version__ = '2.2.1'
logging.basicConfig(format='%(asctime)s %(message)s')


def endpoint(method, endpoint):
    def decorator(func):
        def wrapped(self, *args, **kwargs):
            self.http_method = method
            self.endpoint = endpoint
            return func(self, *args, **kwargs)
        return wrapped
    return decorator


class Api(object):

    POST = 'POST'
    GET = 'GET'
    PUT = 'PUT'
    DELETE = 'DELETE'
    OPTIONS = 'OPTIONS'

    server_uri = ''
    urls = {}
    headers = {}
    endpoint = ''
    http_method = ''

    _api_key = ''
    _api_secret = ''

    def __init__(self, fail_silently=False, timeout_read=15):
        self._session = Session()
        self.timeout_read = timeout_read
        self.fail_silently = fail_silently

    def set_header(self, header):
        """
        :type header: dict
        :return:
        """
        self.headers.update(header)

    def get_headers(self):
        return self.headers

    def query_encode(self, url, payload):
        if '?' not in url:
            url += '?'
        else:
            url += '&'
        for key, value in payload.items():
            url += QueryEncoder.encode_param(key, str(value)) + '&'
        return url.rstrip('&')

    def sign_request(self, auth_timestamp=None):
        request = Request_sig(self.http_method, self.endpoint, {})
        token = Token(self._api_key, self._api_secret)
        auth = request.sign(token, auth_timestamp)
        return auth

    def correct_auth_timestamp(self, msg_error, payload):
        substring = 'Server time: '
        server_timestamp = msg_error[msg_error.index(substring) + len(substring):]
        server_arw = arrow.get(int(server_timestamp))
        right_timestamp = arrow.now().replace(hour=int(server_arw.format('HH'))).timestamp
        return self.request(payload=payload, auth_timestamp=right_timestamp)

    def send_request(self, url, payload):
        if payload and self.http_method == 'GET':
            response = requests.get(url, payload)
        else:
            request = Request(
                self.http_method, url,
                json=payload,
                headers=self.headers
            )
            prepped = request.prepare()
            timeout = (10, self.timeout_read)
            response = self._session.send(prepped, timeout=timeout)

        valid_codes = (200, 201)
        if hasattr(response, 'status'):
            status_code = response.status
        else:
            status_code = response.status_code

        if response and status_code in valid_codes and response.text:
            return json.loads(response.text)
        else:
            logging.info("EXTERNAL API: request error: {0}".format(response.reason))
            if is_json(response.text):
                content = json.loads(response.text)
                msg = content['error'] if 'error' in content else content['detail']
            else:
                msg = response.text[0:response.text.index('Request Method')].strip()
            if 'Timestamp expired' in msg:
                return self.correct_auth_timestamp(msg, payload)
            elif not self.fail_silently:
                raise APIError(message_values=(msg.encode('utf8'),))
            else:
                return {'error': msg.encode('utf8')}

    def request(self, payload=None, auth_timestamp=None):
        url = self.server_uri + self.endpoint
        # Generate and sign request URL
        auth = self.sign_request(auth_timestamp)
        if auth:
            url = self.query_encode(url, auth)

        logging.info('EXTERNAL API: sending request on {0}'.format(url))
        try:
            return self.send_request(url, payload)
        except ReadTimeout as e:
            logging.info("EXTERNAL API: request error: {0}".format(e))
            if not self.fail_silently:
                raise TimeoutError(message_values=(str(self.timeout_read),))
            else:
                return {'error': "The server did not respond within the time you stipulated. "
                                 "The time was {0} second(s)".format(str(self.timeout_read))}
        except (ConnectTimeout, HTTPError) as e:
            logging.info("EXTERNAL API: request error: {0}".format(e))
            if not self.fail_silently:
                raise e
            else:
                return {'error': "{0}".format(e)}

    def sync_request(self, method, url, payload=None, headers=None):
        return self.request(method, url, payload, headers)



import json


def is_json(value):
    try:
        parsed = json.loads(value)
        return True
    except Exception:
        return False


def merge_two_dicts(d1, d2):
    d3 = d1.copy()
    d3.update(d2)
    return d3


def item_in_dict(dictionary, item):
    return item in dictionary and dictionary[item]


def item_not_in_dict(dictionary, item):
    return item not in dictionary or not dictionary[item]


def attr_in_instance(instance, attr):
    return hasattr(instance, attr) and getattr(instance, attr)


def attr_not_in_instance(instance, attr):
    return not hasattr(instance, attr) or not getattr(instance, attr)
